local _,stash = ...
local L = stash.localization
local C = stash.constants
local settings = stash.settings
stash.filterPanel = stash.frame.itemsPanel.Top.FilterPanel

-- the pullout filter panel has different number of columns/rows of dropdowns based on the panel width
-- [pulloutMode] = {minWidth,maxWidth,numColumns,numRows}
stash.filterPanel.configs = {
    [C.FILTER_PANEL_NARROW] = {C.MIN_RESIZE_WIDTH,300,2,3},
    [C.FILTER_PANEL_NORMAL] = {300,600,3,2},
    [C.FILTER_PANEL_WIDE] = {600,C.MAX_RESIZE_WIDTH,6,1}
}

-- ordered list of controls on the filter panel
stash.filterPanel.buttons = {
    stash.filterPanel.LocationDropDown,
    stash.filterPanel.TypeDropDown,
    stash.filterPanel.ResetButton,
    stash.filterPanel.SearchTypesCheckButton,
    stash.filterPanel.CurrentRealmCheckButton,
    stash.filterPanel.SortByRarityCheckButton
}

-- unordered table of drodown menus created during setup, indexed by a menu name (realmMenu, typeMenu, etc)
stash.filterPanel.menus = {}

function stash.filterPanel:Setup()
    self:Resize(true)
    self.ResetButton.Button.Text:SetText(RESET)
    self.SearchTypesCheckButton.Text:SetText(L["Search Types Too"])
    self.CurrentRealmCheckButton.Text:SetText(L["Current Realm & Faction"])
    self.SortByRarityCheckButton.Text:SetText(L["Sort By Rarity"])

    -- create dropdown menus
    self:CreateMenu("realmMenu",self.LocationDropDown,1,C.FILTER_LOCATION_MENU,self.ShowRealmMenu,self.ShowCharacterMenu)
    self:CreateMenu("characterMenu",self.menus.realmMenu,2,C.FILTER_LOCATION_MENU,self.ShowCharacterMenu,self.ShowLocationMenu)
    self:CreateMenu("locationMenu",self.menus.characterMenu,3,C.FILTER_LOCATION_MENU,self.ShowLocationMenu)
    self:CreateMenu("typeMenu",self.TypeDropDown,1,C.FILTER_TYPE_MENU,self.ShowTypeMenu,self.ShowSubTypeMenu)
    self:CreateMenu("subTypeMenu",self.menus.typeMenu,2,C.FILTER_TYPE_MENU,self.ShowSubTypeMenu,self.ShowEquipSlotMenu)
    self:CreateMenu("slotMenu",self.menus.subTypeMenu,3,C.FILTER_TYPE_MENU,self.ShowEquipSlotMenu)
end

-- when filter panel shown, update its contents
function stash.filterPanel:OnShow()
    self:Update()
end

-- resizes and repositions filter panel buttons based on filter panel width
function stash.filterPanel:Resize(force)
    if not force and not self:IsVisible() then
        return -- don't need to do anything if panel isn't on screen
    end
    local panelWidth = self:GetWidth()
    local panelMode
    if panelWidth < self.configs[C.FILTER_PANEL_NARROW][2] then
        panelMode = C.FILTER_PANEL_NARROW
    elseif panelWidth < self.configs[C.FILTER_PANEL_NORMAL][2] then
        panelMode = C.FILTER_PANEL_NORMAL
    else
        panelMode = C.FILTER_PANEL_WIDE
    end

    local cols = self.configs[panelMode][3]
    local rows = self.configs[panelMode][4]

    -- if we're changing to a new panelMode, resize panel and reposition stuff
    if panelMode ~= self.panelMode then
        self.panelMode = panelMode
        local height = rows*C.FILTER_DROPDOWN_HEIGHT+(rows-1)*C.FILTER_DROPDOWN_PADDING
        local padding = C.FILTER_DROPDOWN_PADDING
        self:SetHeight(height)
        self:GetParent():SetHeight(height+32)
        self.LocationDropDown:ClearAllPoints()
        self.TypeDropDown:ClearAllPoints()
        self.ResetButton:ClearAllPoints()
        self.SearchTypesCheckButton:ClearAllPoints()
        self.CurrentRealmCheckButton:ClearAllPoints()
        self.SortByRarityCheckButton:ClearAllPoints()
        if panelMode==C.FILTER_PANEL_NARROW then -- in narrow mode, 2 columns with dropdowns/reset on left, checkbuttons on right
            self.LocationDropDown:SetPoint("TOPLEFT")
            self.TypeDropDown:SetPoint("TOPLEFT",self.LocationDropDown,"BOTTOMLEFT",0,-padding)
            self.ResetButton:SetPoint("TOPLEFT",self.TypeDropDown,"BOTTOMLEFT",0,-padding)
            self.SearchTypesCheckButton:SetPoint("TOPLEFT",self.LocationDropDown,"TOPRIGHT",padding,0)
            self.CurrentRealmCheckButton:SetPoint("TOPLEFT",self.SearchTypesCheckButton,"BOTTOMLEFT",0,-padding)
            self.SortByRarityCheckButton:SetPoint("TOPLEFT",self.CurrentRealmCheckButton,"BOTTOMLEFT",0,-padding)
        elseif panelMode==C.FILTER_PANEL_NORMAL then -- in normal mode, 3 columns with dropdowns/reset across top, checkbuttons across bottom
            self.LocationDropDown:SetPoint("TOPLEFT")
            self.TypeDropDown:SetPoint("TOPLEFT",self.LocationDropDown,"TOPRIGHT",padding,0)
            self.ResetButton:SetPoint("TOPLEFT",self.TypeDropDown,"TOPRIGHT",padding,0)
            self.SearchTypesCheckButton:SetPoint("TOPLEFT",self.LocationDropDown,"BOTTOMLEFT",0,-padding)
            self.CurrentRealmCheckButton:SetPoint("TOPLEFT",self.SearchTypesCheckButton,"TOPRIGHT",padding,0)
            self.SortByRarityCheckButton:SetPoint("TOPLEFT",self.CurrentRealmCheckButton,"TOPRIGHT",padding,0)
        else -- in wide mode, 6 columns with dropdowns first, then checkbuttons, and reset last
            self.LocationDropDown:SetPoint("TOPLEFT")
            self.TypeDropDown:SetPoint("TOPLEFT",self.LocationDropDown,"TOPRIGHT",padding,0)
            self.SearchTypesCheckButton:SetPoint("TOPLEFT",self.TypeDropDown,"TOPRIGHT",padding,0)
            self.CurrentRealmCheckButton:SetPoint("TOPLEFT",self.SearchTypesCheckButton,"TOPRIGHT",padding,0)
            self.SortByRarityCheckButton:SetPoint("TOPLEFT",self.CurrentRealmCheckButton,"TOPRIGHT",padding,0)
            self.ResetButton:SetPoint("TOPLEFT",self.SortByRarityCheckButton,"TOPRIGHT",padding,0)
        end            
    end
    -- whether we changed panelMode or not, resize buttons
    local buttonWidth = (panelWidth - (cols-1))/cols
    for _,button in ipairs(self.buttons) do
        button:SetWidth(buttonWidth)
        if button.CheckButton then
            button.CheckButton:SetHitRectInsets(-1,-buttonWidth+24,-1,-1)
        end
    end
end


-- creates a StashDropDownMenuTemplate and defines attributes passed
function stash.filterPanel:CreateMenu(name,parent,id,menuType,menuFunc,subMenuFunc)
    self.menus[name] = CreateFrame("Frame",nil,parent,"StashDropDownMenuTemplate")
    self.menus[name].menuLevel = id
    self.menus[name].menuType = menuType
    self.menus[name].name = name
    self.menus[name].menuFunc = menuFunc
    self.menus[name].subMenuFunc = subMenuFunc
    parent.subMenu = self.menus[name] -- assigning this new menu as a subMenu of the parent (1:1 relationship)

    if id==1 then -- top-level menus get an OnUpdate to close if the mouse leaves their menus
        self.menus[name].timer = 0
        self.menus[name]:SetScript("OnUpdate",self.MenuOnUpdate)
    end
end


function stash.filterPanel:IsMenuOpen()
    for _,menu in pairs(self.menus) do
        if menu:IsVisible() then
            return true
        end
    end
    return false
end

function stash.filterPanel:CloseMenu()
    for _,menu in pairs(self.menus) do
        if menu:IsVisible() then
            menu:Hide()
        end
    end
end

-- simply sets the text (and color)
function stash.filterPanel:SetDropDownText(dropdown,text)
    dropdown.Text:SetText(text)
    if text==L["Location"] or text==L["Type"] then
        dropdown.Text:SetTextColor(0.6,0.6,0.6)
    else
        dropdown.Text:SetTextColor(1,1,1)
    end
end

-- resets filters to default; if softReset is true then it won't reset checkbuttons
function stash.filterPanel:Reset(softReset)
    stash.itemsPanel:ResetSearch()
    settings.FilterRealm = false
    settings.FilterCharacter = false
    settings.FilterLocation = false
    settings.FilterType = false
    settings.FilterSubType = false
    settings.FilterEquipSlot = false
    if not softReset then
        settings.SearchTypesToo = settings.DefaultSearchTypes
        settings.CurrentRealmOnly = settings.DefaultCurrentRealm
    end
    self:Update()
    stash.tooltips:Pause()
    stash.itemsPanel:UpdateNow()
end

function stash.filterPanel:Update()
    self:SetDropDownText(self.LocationDropDown,settings.FilterLocation or settings.FilterCharacter or settings.FilterRealm or L["Location"])
    self:SetDropDownText(self.TypeDropDown,settings.FilterEquipSlot or settings.FilterSubType or settings.FilterType or L["Type"])
    self.SearchTypesCheckButton.CheckButton:SetChecked(settings.SearchTypesToo)
    self.CurrentRealmCheckButton.CheckButton:SetChecked(settings.CurrentRealmOnly)
    self.SortByRarityCheckButton.CheckButton:SetChecked(settings.SortByRarity)
end

--[[ Menu functions ]]

-- retrieves a menu button for the given menu at the given index; and sets it text and initial state
function stash.filterPanel:GetMenuButton(menu,index,text)
    if not menu.Buttons[index] then
        menu.Buttons[index] = CreateFrame("Button",nil,menu,"StashDropDownMenuLineTemplate")
        menu.Buttons[index]:SetPoint("TOPLEFT",menu.Buttons[index-1],"BOTTOMLEFT")
    end
    menu.Buttons[index].Text:SetText(text)
    menu.Buttons[index].subject = text
    menu.Buttons[index].hasSubMenu = false
    menu.Buttons[index]:Show()
    return menu.Buttons[index]
end

-- resizes a menu based on the given maxWidth of a button and the size of the list
function stash.filterPanel:ResizeMenu(menu,maxWidth,listSize)
    for i=listSize+1,#menu.Buttons do
        menu.Buttons[i]:Hide()
    end
    for i=1,listSize do
        menu.Buttons[i]:SetWidth(maxWidth)
    end
    menu:SetSize(maxWidth+8,listSize*20+8)
end

-- adds the submenu arrow if showArrow is true
function stash.filterPanel:AddSubMenu(button,showArrow)
    button.hasSubMenu = showArrow
    button.Arrow:SetShown(showArrow)
    return showArrow
end

-- does the positioning and display of the menu
function stash.filterPanel:FinishMenu(menu,relativeTo)
    menu.reverseAnchor = false -- assume we're anchoring to right
    if relativeTo then -- for submenus, parent and position to the button the menu opens from
        menu:SetParent(relativeTo)
        menu:ClearAllPoints()
        if relativeTo:GetParent().reverseAnchor or (relativeTo:GetRight()+menu:GetWidth())*relativeTo:GetEffectiveScale() > UIParent:GetRight()*UIParent:GetEffectiveScale() then
            menu.reverseAnchor = true
            menu:SetPoint("TOPRIGHT",relativeTo,"TOPLEFT",1,4)
        else
            menu:SetPoint("TOPLEFT",relativeTo,"TOPRIGHT",-1,4)
        end
    else -- for top-level menus, position to their dropdown parent
        menu:SetPoint("TOPLEFT",menu:GetParent(),"BOTTOMLEFT")
    end
    menu:SetFrameStrata("HIGH")
    menu:Show()
end

-- menu function for the main Type menu
function stash.filterPanel:ShowTypeMenu()
    local list = stash.info:GetTypes()
    if settings.FilterType then -- if a type currently fitlered, add reset option
        tinsert(list,1,C.RESET_MENU_ITEM)
    end
    local maxWidth = 0
    for index,name in ipairs(list) do
        local button = stash.filterPanel:GetMenuButton(self,index,name)
        local textWidth = button.Text:GetStringWidth()+8
        if stash.filterPanel:AddSubMenu(button,stash.info:HasTypes(name)) then
            textWidth = textWidth + 14
        end
        maxWidth = max(maxWidth,textWidth)
    end

    stash.filterPanel:ResizeMenu(self,maxWidth,#list)
    stash.filterPanel:FinishMenu(self)
end

-- menu function for the SubType menu
function stash.filterPanel:ShowSubTypeMenu(relativeTo)
    self.subMenu:Hide()
    local list = stash.info:GetTypes(relativeTo.subject)
    local maxWidth = 0
    for index,name in ipairs(list) do
        local button = stash.filterPanel:GetMenuButton(self,index,name)
        local textWidth = button.Text:GetStringWidth()+8
        if stash.filterPanel:AddSubMenu(button,stash.info:HasTypes(relativeTo.subject,name)) then
            textWidth = textWidth + 14
        end
        maxWidth = max(maxWidth,textWidth)
    end
    stash.filterPanel:ResizeMenu(self,maxWidth,#list)
    stash.filterPanel:FinishMenu(self,relativeTo)
end

-- menu function for the EquipSlot menu
function stash.filterPanel:ShowEquipSlotMenu(relativeTo)
    local list = stash.info:GetTypes(relativeTo:GetParent():GetParent().subject,relativeTo.subject)
    local maxWidth = 0
    for index,name in ipairs(list) do
        local button = stash.filterPanel:GetMenuButton(self,index,name)
        local textWidth = button.Text:GetStringWidth()+8
        stash.filterPanel:AddSubMenu(button,false)
        maxWidth = max(maxWidth,textWidth)
    end
    stash.filterPanel:ResizeMenu(self,maxWidth,#list)
    stash.filterPanel:FinishMenu(self,relativeTo)
end

-- menu function for main Locations menu (actually realm)
function stash.filterPanel:ShowRealmMenu()
    local list = stash.info:GetLocations()
    if settings.FilterRealm then -- if realm currently filtered, add reset option
        tinsert(list,1,C.RESET_MENU_ITEM)
    end
    local maxWidth = 0
    for index,name in ipairs(list) do
        local button = stash.filterPanel:GetMenuButton(self,index,name)
        local textWidth = button.Text:GetStringWidth()+8+14
        stash.filterPanel:AddSubMenu(button,name~=C.RESET_MENU_ITEM)
        maxWidth = max(maxWidth,textWidth)
    end
    stash.filterPanel:ResizeMenu(self,maxWidth,#list)
    stash.filterPanel:FinishMenu(self)
end

-- menu function for character submenu
function stash.filterPanel:ShowCharacterMenu(relativeTo)
    self.subMenu:Hide()
    local list = stash.info:GetLocations(relativeTo.subject)
    local maxWidth = 0
    for index,name in ipairs(list) do
        local button = stash.filterPanel:GetMenuButton(self,index,name)
        local textWidth = button.Text:GetStringWidth()+8+14
        stash.filterPanel:AddSubMenu(button,true)
        maxWidth = max(maxWidth,textWidth)
    end
    stash.filterPanel:ResizeMenu(self,maxWidth,#list)
    stash.filterPanel:FinishMenu(self,relativeTo)
end

-- menu function for the Location menu
function stash.filterPanel:ShowLocationMenu(relativeTo)
    local list = stash.info:GetLocations(relativeTo:GetParent():GetParent().subject,relativeTo.subject)
    local maxWidth = 0
    for index,name in ipairs(list) do
        local button = stash.filterPanel:GetMenuButton(self,index,name)
        local textWidth = button.Text:GetStringWidth()+8
        stash.filterPanel:AddSubMenu(button,false)
        maxWidth = max(maxWidth,textWidth)
    end
    stash.filterPanel:ResizeMenu(self,maxWidth,#list)
    stash.filterPanel:FinishMenu(self,relativeTo)
end

--[[ DropDown/Menu script handlers ]]

-- top-level menus will wait two seconds (C.MENU_CLOSE_DELAY) after the mouse has left the menus before closing them
-- (note: this assumes menus have 3 levels: self, self.subMenu and self.subMenu.subMenu)
function stash.filterPanel:MenuOnUpdate(elapsed)
    local lastMenu
    if MouseIsOver(self:GetParent()) or MouseIsOver(self) or MouseIsOver(self.subMenu) or MouseIsOver(self.subMenu.subMenu) then
        self.timer = 0
    else
        self.timer = self.timer + elapsed
    end
    if self.timer > C.MENU_CLOSE_DELAY then
        stash.filterPanel:CloseMenu()
    end
end

-- click of one of the two dropdowns (note that self is stash.filterPanel and not the clicked dropdown--which is the dropdown arg)
function stash.filterPanel:DropDownOnClick(dropdown)
    if dropdown==self.LocationDropDown then -- toggling location dropdown menu
        self.menus.typeMenu:Hide()
        if self.menus.realmMenu:IsVisible() then
            self.menus.realmMenu:Hide()
        else
            stash.filterPanel.ShowRealmMenu(self.menus.realmMenu)
        end
    else -- toggling type dropdown menu
        self.menus.realmMenu:Hide()
        if self.menus.typeMenu:IsVisible() then
            self.menus.typeMenu:Hide()
        else
            stash.filterPanel.ShowTypeMenu(self.menus.typeMenu)
        end
    end
end

-- mouse entering a menu button (may potentially open a submenu)
function stash.filterPanel:MenuButtonOnEnter()
    self.Highlight:Show()
    if self.hasSubMenu and self:GetParent().subMenuFunc then
        self:GetParent().subMenuFunc(self:GetParent().subMenu,self)
    end
end

-- mouse leaving a menu button
function stash.filterPanel:MenuButtonOnLeave()
    self.Highlight:Hide()
    if self:GetParent().menuLevel<3 and not MouseIsOver(self:GetParent().subMenu) then
        self:GetParent().subMenu:Hide()
    end
end

-- clicking a menu button
function stash.filterPanel:MenuButtonOnClick()
    local menuLevel = self:GetParent().menuLevel
    local menuType = self:GetParent().menuType
    local setting1,setting2,setting3 -- the 3 tiers of settings

    -- if choosing the inserted "Reset" option for a top-level menu, reset 
    if menuLevel==1 and self.subject==C.RESET_MENU_ITEM then
        setting1 = false
        setting2 = false
        setting3 = false
    elseif menuLevel==1 then -- choosing a non-Reset top-level menu (Realm/Type)
        setting1 = self.subject
        setting2 = false
        setting3 = false
    elseif menuLevel==2 then -- choosing a secondary menu (Character/SubType)
        setting1 = self:GetParent():GetParent().subject
        setting2 = self.subject
        setting3 = false
    elseif menuLevel==3 then -- choosing a tertiary menu (Location/EquipSlot)
        setting1 = self:GetParent():GetParent():GetParent():GetParent().subject
        setting2 = self:GetParent():GetParent().subject
        setting3 = self.subject
    end
    if menuType==C.FILTER_LOCATION_MENU then -- set filter savedvars for location dropdown
        settings.FilterRealm = setting1
        settings.FilterCharacter = setting2
        settings.FilterLocation = setting3
        -- if CurrentRealmOnly enabled and this filter conflicts with current realm/faction, un-enable CurrentRealmOnly
        if settings.CurrentRealmOnly and ((setting1 and setting1~=stash.current.realm) or (setting1 and setting2 and stash.info:GetFaction(setting1,setting2)~=stash.current.faction)) then
            settings.CurrentRealmOnly = false
        end
    elseif menuType==C.FILTER_TYPE_MENU then -- set filter savedvars for type dropdown
        settings.FilterType = setting1
        settings.FilterSubType = setting2
        settings.FilterEquipSlot = setting3
    end
    stash.filterPanel:Update()
    stash.filterPanel:CloseMenu()
    stash.itemsPanel:UpdateNow()
end

-- clicking a checkbutton; the savedvar it changes is a keyvalue in the xml
function stash.filterPanel:CheckButtonOnClick()
    local var = self:GetParent().var
    settings[var] = self:GetChecked()

    -- if we've enabled CurrentRealmOnly and we're currently filtered to a realm or faction outside current, then clear filter
    if var=="CurrentRealmOnly" and settings.CurrentRealmOnly and ((settings.FilterRealm and settings.FilterRealm~=stash.current.realm) or (settings.FilterRealm and settings.FilterCharacter and stash.info:GetFaction(settings.FilterRealm,settings.FilterCharacter)~=stash.curren.faction)) then
        stash.settings.FilterRealm = false
        stash.settings.FilterCharacter = false
        stash.settings.FilterLocation = false
    end

    stash.filterPanel:Update()
    stash.itemsPanel:UpdateNow()
end

-- returns a list of active filters to be displayed in results "Search, Hyjal, Gello, Weapons"
local filterList = {} -- reusing to minimize garbage
local filterOrder = {"FilterRealm","FilterCharacter","FilterLocation","FilterType","FilterSubType","FilterEquipSlot"}
function stash.filterPanel:GetActiveFilterList()
    wipe(filterList)
    for _,filter in ipairs(filterOrder) do
        if settings[filter] then
            tinsert(filterList,settings[filter])
        end
    end
    if stash.info:IsSearching() then
        tinsert(filterList,L["Search"])
    end
    return table.concat(filterList,", ")
end